HOWTO: add missing [netsplice] for ioengine specific options
[fio.git] / file.h
diff --git a/file.h b/file.h
index 994d5fd0c828bd8de6c6c39bce44e42d45e11d8f..84daa5f66b74a1c871796dd20c1f3375e0eacf89 100644 (file)
--- a/file.h
+++ b/file.h
@@ -1,14 +1,21 @@
 #ifndef FIO_FILE_H
 #define FIO_FILE_H
 
+#include <string.h>
+#include "compiler/compiler.h"
 #include "io_ddir.h"
+#include "flist.h"
+#include "lib/zipf.h"
+#include "lib/axmap.h"
+#include "lib/lfsr.h"
+#include "lib/gauss.h"
 
 /*
  * The type of object we are working on
  */
 enum fio_filetype {
        FIO_TYPE_FILE = 1,              /* plain file */
-       FIO_TYPE_BD,                    /* block device */
+       FIO_TYPE_BLOCK,                 /* block device */
        FIO_TYPE_CHAR,                  /* character device */
        FIO_TYPE_PIPE,                  /* pipe */
 };
@@ -21,6 +28,8 @@ enum fio_file_flags {
        FIO_FILE_size_known     = 1 << 4,       /* size has been set */
        FIO_FILE_hashed         = 1 << 5,       /* file is on hash */
        FIO_FILE_partial_mmap   = 1 << 6,       /* can't do full mmap */
+       FIO_FILE_axmap          = 1 << 7,       /* uses axmap */
+       FIO_FILE_lfsr           = 1 << 8,       /* lfsr is used */
 };
 
 enum file_lock_mode {
@@ -30,13 +39,31 @@ enum file_lock_mode {
 };
 
 /*
- * roundrobin available files, or choose one at random, or do each one
- * serially.
+ * How fio chooses what file to service next. Choice of uniformly random, or
+ * some skewed random variants, or just sequentially go through them or
+ * roundrobing.
  */
 enum {
-       FIO_FSERVICE_RANDOM     = 1,
-       FIO_FSERVICE_RR         = 2,
-       FIO_FSERVICE_SEQ        = 3,
+       FIO_FSERVICE_RANDOM             = 1,
+       FIO_FSERVICE_RR                 = 2,
+       FIO_FSERVICE_SEQ                = 3,
+       __FIO_FSERVICE_NONUNIFORM       = 0x100,
+       FIO_FSERVICE_ZIPF               = __FIO_FSERVICE_NONUNIFORM | 4,
+       FIO_FSERVICE_PARETO             = __FIO_FSERVICE_NONUNIFORM | 5,
+       FIO_FSERVICE_GAUSS              = __FIO_FSERVICE_NONUNIFORM | 6,
+
+       FIO_FSERVICE_SHIFT              = 10,
+};
+
+/*
+ * No pre-allocation when laying down files, or call posix_fallocate(), or
+ * call fallocate() with FALLOC_FL_KEEP_SIZE set.
+ */
+enum fio_fallocate_mode {
+       FIO_FALLOCATE_NONE      = 1,
+       FIO_FALLOCATE_POSIX     = 2,
+       FIO_FALLOCATE_KEEP_SIZE = 3,
+       FIO_FALLOCATE_NATIVE    = 4,
 };
 
 /*
@@ -47,9 +74,9 @@ struct fio_file {
        struct flist_head hash_list;
        enum fio_filetype filetype;
 
-       void *file_data;
        int fd;
-#ifdef __CYGWIN__
+       int shadow_fd;
+#ifdef WIN32
        HANDLE hFile;
        HANDLE ioCP;
 #endif
@@ -57,45 +84,66 @@ struct fio_file {
        /*
         * filename and possible memory mapping
         */
-       char *file_name;
        unsigned int major, minor;
-
-       void *mmap_ptr;
-       size_t mmap_sz;
-       off_t mmap_off;
+       int fileno;
+       int bs;
+       char *file_name;
 
        /*
         * size of the file, offset into file, and io size from that offset
+        * (be aware io_size is different from thread_options::io_size)
+        */
+       uint64_t real_file_size;
+       uint64_t file_offset;
+       uint64_t io_size;
+
+       /*
+        * Track last end and last start of IO for a given data direction
         */
-       unsigned long long real_file_size;
-       unsigned long long file_offset;
-       unsigned long long io_size;
+       uint64_t last_pos[DDIR_RWDIR_CNT];
+       uint64_t last_start[DDIR_RWDIR_CNT];
 
-       unsigned long long last_pos;
-       unsigned long long last_start;
+       uint64_t first_write;
+       uint64_t last_write;
 
-       unsigned long long first_write;
-       unsigned long long last_write;
+       /*
+        * Tracks the last iodepth number of completed writes, if data
+        * verification is enabled
+        */
+       uint64_t *last_write_comp;
+       unsigned int last_write_idx;
 
        /*
-        * For use by the io engine
+        * For use by the io engine for offset or private data storage
         */
-       unsigned long long file_pos;
+       union {
+               uint64_t engine_pos;
+               void *engine_data;
+       };
 
        /*
         * if io is protected by a semaphore, this is set
         */
-       struct fio_mutex *lock;
-       void *lock_owner;
-       unsigned int lock_batch;
-       enum fio_ddir lock_ddir;
+       union {
+               struct fio_mutex *lock;
+               struct fio_rwlock *rwlock;
+       };
 
        /*
* block map for random io
         */
-       unsigned int *file_map;
-       unsigned int num_maps;
-       unsigned int last_free_lookup;
+       union {
+               struct axmap *io_axmap;
+               struct fio_lfsr lfsr;
+       };
+
+       /*
+        * Used for zipf random distribution
+        */
+       union {
+               struct zipf_state zipf;
+               struct gauss_state gauss;
+       };
 
        int references;
        enum fio_file_flags flags;
@@ -103,14 +151,17 @@ struct fio_file {
        struct disk_util *du;
 };
 
+#define FILE_ENG_DATA(f)               ((f)->engine_data)
+#define FILE_SET_ENG_DATA(f, data)     ((f)->engine_data = (data))
+
 #define FILE_FLAG_FNS(name)                                            \
 static inline void fio_file_set_##name(struct fio_file *f)             \
 {                                                                      \
-       (f)->flags |= FIO_FILE_##name;                                  \
+       (f)->flags = (enum fio_file_flags) ((f)->flags | FIO_FILE_##name);      \
 }                                                                      \
 static inline void fio_file_clear_##name(struct fio_file *f)           \
 {                                                                      \
-       (f)->flags &= ~FIO_FILE_##name;                                 \
+       (f)->flags = (enum fio_file_flags) ((f)->flags & ~FIO_FILE_##name);     \
 }                                                                      \
 static inline int fio_file_##name(struct fio_file *f)                  \
 {                                                                      \
@@ -124,6 +175,8 @@ FILE_FLAG_FNS(done);
 FILE_FLAG_FNS(size_known);
 FILE_FLAG_FNS(hashed);
 FILE_FLAG_FNS(partial_mmap);
+FILE_FLAG_FNS(axmap);
+FILE_FLAG_FNS(lfsr);
 #undef FILE_FLAG_FNS
 
 /*
@@ -132,13 +185,16 @@ FILE_FLAG_FNS(partial_mmap);
 struct thread_data;
 extern void close_files(struct thread_data *);
 extern void close_and_free_files(struct thread_data *);
+extern uint64_t get_start_offset(struct thread_data *, struct fio_file *);
 extern int __must_check setup_files(struct thread_data *);
 extern int __must_check file_invalidate_cache(struct thread_data *, struct fio_file *);
 extern int __must_check generic_open_file(struct thread_data *, struct fio_file *);
 extern int __must_check generic_close_file(struct thread_data *, struct fio_file *);
 extern int __must_check generic_get_file_size(struct thread_data *, struct fio_file *);
+extern int __must_check file_lookup_open(struct fio_file *f, int flags);
 extern int __must_check pre_read_files(struct thread_data *);
-extern int add_file(struct thread_data *, const char *);
+extern unsigned long long get_rand_file_size(struct thread_data *td);
+extern int add_file(struct thread_data *, const char *, int, int);
 extern int add_file_exclusive(struct thread_data *, const char *);
 extern void get_file(struct fio_file *);
 extern int __must_check put_file(struct thread_data *, struct fio_file *);
@@ -151,15 +207,9 @@ extern int init_random_map(struct thread_data *);
 extern void dup_files(struct thread_data *, struct thread_data *);
 extern int get_fileno(struct thread_data *, const char *);
 extern void free_release_files(struct thread_data *);
-
-static inline void fio_file_reset(struct fio_file *f)
-{
-       f->last_free_lookup = 0;
-       f->last_pos = f->file_offset;
-       f->last_start = -1ULL;
-       f->file_pos = -1ULL;
-       if (f->file_map)
-               memset(f->file_map, 0, f->num_maps * sizeof(int));
-}
+extern void filesetup_mem_free(void);
+extern void fio_file_reset(struct thread_data *, struct fio_file *);
+extern bool fio_files_done(struct thread_data *);
+extern bool exists_and_not_regfile(const char *);
 
 #endif
@@ -57,45 +84,66 @@ struct fio_file {
 
        /*
-        * block map for random io
+        * block map or LFSR for random io
         */